/* Process a specific DLL */
parse upper arg FileToProcess SysOpen
    if FileToProcess <> '' then do
        '@echo off'
        PrevDir = Directory();
        DLLOpen = LinkFiles(SysOpen);
        Home = SysIni('USER', 'NewWarp', 'Home');
        Boot = SysIni('USER', 'NewWarp', 'Boot');
        if (Home <> '') & (Boot <> '') then do
            if Directory(Home) <> '' then do
                rc = SysMkDir('Backup-DLL');
                if (rc = 0) | (rc = 5) then do
                    if Directory(FileToProcess) <> '' then do
                        say 'Processing the file 'FileToProcess'.DLL';
                        if StepOne(Boot, Home, FileToProcess) = 'ERROR' then do
                            FileToProcess = 'ERROR';
                        end
                        else do
                            rc = StepTwo(Boot, FileToProcess);
                            rc = StepThree(Home, FileToProcess);
                            if StepFour(Boot, Home, FileToProcess) = 'ERROR' then do
                                rc = DoCleanUp(DLLOpen, PrevDir);
                                FileToProcess = 'ERROR';
                            end
                            rc = Directory(Home);
                            say 'Finished processing 'FileToProcess'.DLL!'
                        end
                    end
                    else do
                        say 'ERROR: Could not change to the directory 'Home'\'FileToProcess'!!!'
                        say
                        say 'Please check that the directory exists!!'
                        FileToProcess = 'ERROR';
                    end
                end
                else do
                    say 'ERROR: Could not create or open backup directory!!';
                    say
                    say 'This directory must be created as a subdirectory of 'Home', please verify'
                    say 'that this can be done'
                    say
                    say 'If after verifying the above is possible the error persists, re-run Install.cmd'
                    FileToProcess = 'ERROR';
                end
            end
            else do
                say 'ERROR: Could not change to the directory 'Home'!!!'
                say
                say 'Re-run INSTALL.CMD from the installation directory to repair the INI values'
                FileToProcess = 'ERROR';
            end
        end
        else do
            say "Install.cmd has not been run!! Please run this to be able to use NewWarp!!"
            FileToProcess = 'ERROR';
        end
        rc = DoCleanUp(DLLOpen, PrevDir);
    end
    else do
        say 'Syntax is: PROCESS [File]'
        say
        say 'Where [File] is the filename (without path or extension) of the DLL to be processed.'
    end
return FileToProcess

LinkFiles: procedure
    arg SysOpen
    if SysOpen = '' then do
        DLLOpen = 0;
        PrevDir = Directory();
        Add = RxFuncAdd('SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs');
        if Add = 0 then do
            rc = SysLoadFuncs();
        end
        else do
            DLLOpen = DLLOpen + 1;
        end
        Add = RxFuncAdd('FileLoadFuncs', 'FILEREXX', 'FileLoadFuncs');
        if Add = 0 then do
            rc = FileLoadFuncs();
        end
        else do
            DLLOpen = DLLOpen + 2;
        end
    end
    else do
        DLLOpen = 3;
    end
return DLLOpen;

DoCleanup: procedure
    arg DLLOpen, PrevDir
        select
            when DLLOpen = 0 then do
                    rc = FileDropFuncs();
                    rc = SysDropFuncs();
                end
            when DLLOpen = 1 then do
                    rc = FileDropFuncs();
                end
            when DLLOpen = 2 then do
                    rc = SysDropFuncs();
                end
            otherwise do
                    rc = 0;
                end
        end
        rc = Directory(PrevDir);
return ''

StepOne: procedure
    arg Boot, Home, FileToProcess
    say 'Step 1: Backup and Copy Original 'FileToProcess'.DLL'
    if FileCopyFile(Boot':\os2\dll\'FileToProcess'.dll', Home'\Backup-DLL', 'REPLACE') <> 0 then do
        say 'FATAL ERROR: Could not copy 'Boot':\os2\dll\'FileToProcess'.dll for '
        say '             further processing!!!'
        say
        say 'Possible causes are:'
        say '    -    Directory 'Home'\Backup-DLL does not exist or is not'
        say '         accessible'
        say '    -    File 'Boot':\os2\dll\'FileToProcess'.dll does not exist'
        say '         or is inaccessible'
        say
        say 'Check these conditions and try again'
        FileToProcess = 'ERROR'
    end
    else do
        rc = FileCopyFile(Boot':\os2\dll\'FileToProcess'.dll', Directory(), 'REPLACE');
    end
return FileToProcess

StepTwo: procedure
    arg Boot, FileToProcess
    say 'Step 2: Compiling new Resources'
    Boot':\os2\rc -r 'FileToProcess'.rc2 >nul'
return FileToProcess

StepThree: procedure
    arg Home, FileToProcess
    say 'Step 3: Building New Version of 'FileToProcess'.DLL'
    'call 'Home'\resmgr -A 'FileToProcess'.dll 'FileToProcess'.res >nul'
return FileToProcess

StepFour: procedure
    arg Boot, Home, FileToProcess
    say 'Step 4: Replacing 'FileToProcess'.DLL in 'Boot':\os2\dll'
    Home'\unlock 'Boot':\os2\dll\'FileToProcess'.dll >nul'
    if FileCopyFile(FileToProcess'.dll', Boot':\os2\dll', 'REPLACE') <> 0 then do
        say '     -     Unable to copy 'FileToProcess'.DLL to 'Boot':\os2\dll\'FileToProcess'.DLL'
        say '     -     Refer to the Readme file for instructions on MANUAL REPLACEMENT'
    end
    if FileDeleteFile(FileToProcess'.res') <> 0 then do
        Say '     -     NonFatal: Unable to delete temporary file: 'FileToProcess'.res,'
        say '     -               Please delete the file manually after the processing'
        say '     -               has completed.'
    end
    else do
        rc = StepFive(FileToProcess);
    end
return FileToProcess

StepFive: procedure
    arg FileToProcess
    say 'Step 5: Deleting temporary files'
    if FileDeleteFile(FileToProcess'.dll') <> 0 then do
        Say '     -     NonFatal: Unable to delete temporary file: 'FileToProcess'.dll'
        say '                     You may delete this file manually after the processing has'
        say '                     completed.'
    end
return FileToProcess
